@@ -1,15 +1,14 @@
-use std::{cell::RefCell, future::Future, rc::Rc};
+use std::future::Future;
 
-use ntex::codec::{AsyncRead, AsyncWrite};
 use ntex::connect::{self, Address, Connect, Connector};
-use ntex::framed::{ReadTask, State, WriteTask};
-use ntex::{service::Service, time::Seconds, util::ByteString};
+use ntex::io::{utils::Boxed, IoBoxed};
+use ntex::{service::Service, time::Seconds, util::ByteString, util::PoolId, util::PoolRef};
 
 #[cfg(feature = "openssl")]
-use ntex::connect::openssl::{OpensslConnector, SslConnector};
+use ntex::connect::openssl::{self, SslConnector};
 
 #[cfg(feature = "rustls")]
-use ntex::connect::rustls::{ClientConfig, RustlsConnector};
+use ntex::connect::rustls::{self, ClientConfig};
 
 use super::errors::ConnectError;
 use super::{cmd, Client, SimpleClient};
@@ -19,9 +18,7 @@ pub struct RedisConnector<A, T> {
     address: A,
     connector: T,
     passwords: Vec<ByteString>,
-    lw: u16,
-    read_hw: u16,
-    write_hw: u16,
+    pool: PoolRef,
 }
 
 impl<A> RedisConnector<A, ()>
@@ -30,24 +27,17 @@ where
 {
     #[allow(clippy::new_ret_no_self)]
     /// Create new redis connector
-    pub fn new(address: A) -> RedisConnector<A, Connector<A>> {
+    pub fn new(address: A) -> RedisConnector<A, Boxed<Connector<A>, Connect<A>>> {
         RedisConnector {
             address,
             passwords: Vec::new(),
-            connector: Connector::default(),
-            lw: 1024,
-            read_hw: 16 * 1024,
-            write_hw: 16 * 1024,
+            connector: Connector::default().seal(),
+            pool: PoolId::P7.pool_ref(),
         }
     }
 }
 
-impl<A, T> RedisConnector<A, T>
-where
-    A: Address + Clone,
-    T: Service<Request = Connect<A>, Error = connect::ConnectError>,
-    T::Response: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+impl<A, T> RedisConnector<A, T> {
     /// Add redis auth password
     pub fn password<U>(mut self, password: U) -> Self
     where
@@ -58,48 +48,53 @@ where
         self
     }
 
-    #[inline]
-    /// Set read/write buffer params
+    /// Set memory pool.
     ///
-    /// By default read buffer is 16kb, write buffer is 16kb
-    pub fn buffer_params(
-        mut self,
-        max_read_buf_size: u16,
-        max_write_buf_size: u16,
-        min_buf_size: u16,
-    ) -> Self {
-        self.read_hw = max_read_buf_size;
-        self.write_hw = max_write_buf_size;
-        self.lw = min_buf_size;
+    /// Use specified memory pool for memory allocations. By default P7
+    /// memory pool is used.
+    pub fn memory_pool(mut self, id: PoolId) -> Self {
+        self.pool = id.pool_ref();
         self
     }
 
     /// Use custom connector
-    pub fn connector<U>(self, connector: U) -> RedisConnector<A, U>
+    pub fn connector<Io, U>(self, connector: U) -> RedisConnector<A, Boxed<U, Connect<A>>>
     where
-        U: Service<Request = Connect<A>, Error = connect::ConnectError>,
-        U::Response: AsyncRead + AsyncWrite + Unpin + 'static,
+        U: Service<Connect<A>, Response = Io, Error = connect::ConnectError>,
+        IoBoxed: From<Io>,
+    {
+        RedisConnector {
+            connector: Boxed::new(connector),
+            address: self.address,
+            passwords: self.passwords,
+            pool: self.pool,
+        }
+    }
+
+    /// Use custom boxed connector
+    pub fn boxed_connector<U>(self, connector: U) -> RedisConnector<A, U>
+    where
+        U: Service<Connect<A>, Response = IoBoxed, Error = connect::ConnectError>,
     {
         RedisConnector {
             connector,
             address: self.address,
             passwords: self.passwords,
-            read_hw: self.read_hw,
-            write_hw: self.write_hw,
-            lw: self.lw,
+            pool: self.pool,
         }
     }
 
     #[cfg(feature = "openssl")]
     /// Use openssl connector.
-    pub fn openssl(self, connector: SslConnector) -> RedisConnector<A, OpensslConnector<A>> {
+    pub fn openssl(
+        self,
+        connector: SslConnector,
+    ) -> RedisConnector<A, Boxed<openssl::Connector<A>, Connect<A>>> {
         RedisConnector {
             address: self.address,
             passwords: self.passwords,
-            connector: OpensslConnector::new(connector),
-            read_hw: self.read_hw,
-            write_hw: self.write_hw,
-            lw: self.lw,
+            connector: Boxed::new(openssl::Connector::new(connector)),
+            pool: self.pool,
         }
     }
 
@@ -107,35 +102,34 @@ where
     /// Use rustls connector.
     pub fn rustls(
         self,
-        config: std::sync::Arc<ClientConfig>,
-    ) -> RedisConnector<A, RustlsConnector<A>> {
+        config: ClientConfig,
+    ) -> RedisConnector<A, Boxed<rustls::Connector<A>, Connect<A>>> {
         RedisConnector {
             address: self.address,
             passwords: self.passwords,
-            connector: RustlsConnector::new(config),
-            read_hw: self.read_hw,
-            write_hw: self.write_hw,
-            lw: self.lw,
+            connector: Boxed::new(rustls::Connector::new(config)),
+            pool: self.pool,
         }
     }
+}
 
+impl<A, T> RedisConnector<A, T>
+where
+    A: Address + Clone,
+    T: Service<Connect<A>, Response = IoBoxed, Error = connect::ConnectError>,
+{
     /// Connect to redis server and create shared client
     pub fn connect(&self) -> impl Future<Output = Result<Client, ConnectError>> {
-        let read_hw = self.read_hw;
-        let write_hw = self.write_hw;
-        let lw = self.lw;
+        let pool = self.pool;
         let passwords = self.passwords.clone();
         let fut = self.connector.call(Connect::new(self.address.clone()));
 
         async move {
             let io = fut.await?;
+            io.set_memory_pool(pool);
+            io.set_disconnect_timeout(Seconds::ZERO.into());
 
-            let state = State::with_params(read_hw, write_hw, lw, Seconds::ZERO);
-            let io = Rc::new(RefCell::new(io));
-            ntex::rt::spawn(ReadTask::new(io.clone(), state.clone()));
-            ntex::rt::spawn(WriteTask::new(io, state.clone()));
-
-            let client = Client::new(state);
+            let client = Client::new(io);
 
             if passwords.is_empty() {
                 Ok(client)
@@ -152,21 +146,16 @@ where
 
     /// Connect to redis server and create simple client
     pub fn connect_simple(&self) -> impl Future<Output = Result<SimpleClient, ConnectError>> {
-        let read_hw = self.read_hw;
-        let write_hw = self.write_hw;
-        let lw = self.lw;
+        let pool = self.pool;
         let passwords = self.passwords.clone();
         let fut = self.connector.call(Connect::new(self.address.clone()));
 
         async move {
             let io = fut.await?;
+            io.set_memory_pool(pool);
+            io.set_disconnect_timeout(Seconds::ZERO.into());
 
-            let state = State::with_params(read_hw, write_hw, lw, Seconds::ZERO);
-            let io = Rc::new(RefCell::new(io));
-            ntex::rt::spawn(ReadTask::new(io.clone(), state.clone()));
-            ntex::rt::spawn(WriteTask::new(io, state.clone()));
-
-            let mut client = SimpleClient::new(state);
+            let client = SimpleClient::new(io);
 
             if passwords.is_empty() {
                 Ok(client)
